import pygame
import sys
import map
import weakref
import model.images
import model.sound
import model.tank
import model.fire
import model.wall
import model.enemy


class Controller(object):

    def __init__(self, map_filename):
        self._map = map.Map(map_filename)
        self._map_bg, self._bg_rect = model.images.get_image('background')
        self._player = self._map.sprites[model.tank.Tank].sprites()[0]
        for group in self._map.sprites.itervalues():
            for sprite in group:
                if hasattr(sprite, 'set_walls'):
                    sprite.set_walls(self._map.sprites[model.wall.Wall])
        self._clock = pygame.time.Clock()
        self._music = model.sound.get_sound('music')
        self._music.play(-1)
        self._fires = pygame.sprite.RenderUpdates()
        self.__pits = self._map.sprites[model.pit.Pit]

    def run(self, screen):
        while True:
            #get pressed keys
            keys = pygame.key.get_pressed()
            #check for Alt-F4
            if ((keys[pygame.K_RALT] or keys[pygame.K_LALT]) and keys[pygame.K_F4]) or keys[pygame.K_ESCAPE]:
                sys.exit()
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    sys.exit()
                elif (event.type == pygame.KEYDOWN and
                            (event.key == pygame.K_w
                                or event.key == pygame.K_a
                                or event.key == pygame.K_s
                                or event.key == pygame.K_d)):
                    self._player.charge()
                elif (event.type == pygame.KEYUP and
                            (event.key == pygame.K_w
                                or event.key == pygame.K_a
                                or event.key == pygame.K_s
                                or event.key == pygame.K_d)):
                    charge_strength = self._player.get_charge()
                    if charge_strength < 10:
                        self._player.clear_charge()
                        self._player.turn_right()
                    else:
                        self._player.fire()
                        self._fires.add( model.fire.Fire(self._player.rect.center, self._player.get_direction(), charge_strength) )
            screen.blit(self._map_bg, (0, 0))
            #Fire detect walls
            for fire in self._fires:
                collisions = pygame.sprite.spritecollide(
                    fire, self._map.sprites[model.enemy.Enemy], False)
                for enemy in collisions:
                    enemy.die_sound.play()
                    self._map.sprites[model.enemy.Enemy].remove(enemy)
                collisions = pygame.sprite.spritecollide(
                    fire, self._map.sprites[model.wall.Wall], False)
                if collisions:
                    for wall in collisions:
                        wall.damage(fire.get_strength())
                    self._fires.remove(fire)
            for group in self._map.sprites.itervalues():
                group.update()
                group.draw(screen)
            shot = pygame.sprite.spritecollide(self._player, self._map.sprites[model.enemy.Enemy], False)
            if shot and self._player.shot(10):
                print "I'm dead!"
                sys.exit()
            gameover = pygame.sprite.spritecollide(self._player, self.__pits, False)
            if gameover:
                print "I'm in a pit!"
                sys.exit()
            self._fires.update()
            self._fires.draw(screen)
            pygame.display.flip()
            self._clock.tick(30)
    
